#!/bin/bash

# Scorelog converter for freeciv (ver >=1.14.1) !
# ver 1.5
# Written by Dymitr Pszenicyn
# e-mail: d.pszenicyn@students.mimuw.edu.pl


# Some functions:

# Args: (in_file)
get_max_tag_num(){
    declare -i RET_VAL
    RET_VAL=`cat $1 | grep "tag [^ ]* .*" | cut -f2 -d " " | sort -n | uniq | tail -n 1`
    return $RET_VAL;
}

# Args: (in_file, tag_name)
get_tag_by_name(){
    declare -i RET_VAL
    RET_VAL=`cat $1 | grep "tag [^ ]* $2\$" | cut -f 2 -d " "`
    return $RET_VAL;
}

# Args: (in_file, tag_num)
get_tag_by_num(){
    RET_VAL=`cat $1 | grep "tag $2 .*" | cut -f 3 -d " "`
}

# Args: (in_file, player_num)
get_player_name_by_num(){
    RET_VAL=`cat $1 | grep "addplayer [^ ]* $2 .*" | cut -f4- -d " " | sort | uniq | tr "\n" " "`
}

# Args: (in_file)
get_max_player_num(){
    declare -i RET_VAL
    RET_VAL=`cat $1 | grep "addplayer [^ ]* .*" | cut -f3 -d " " | sort | uniq | tail -n 1`
    return $RET_VAL;
}

# Args: (in_file, tag_num, tag_name)
create_txt_files(){
    for i in `seq 0 $MAX_PLAYER`
    do
	cat $1 | grep "data [^ ]* $2 $i .*" >fcivl_$3_$i.txt
    done
}

# Args: (out_file, string)
fprint(){
    echo -e -n "$2" >>$1
}

# Args: (out_file)
fwrite_gpl_head(){
    fprint $1 "# This is a gnuplot readable file. \n"
    fprint $1 "set terminal x11 \n"
    fprint $1 "set data style lines \n"
    fprint $1 "set xlabel \"time [turns]\" \n"
    fprint $1 "set ylabel \"value\" \n"
    fprint $1 "\n"
}

# Args: (tag_num, max_player, in_file)
create_gpl_file(){
    get_tag_by_num $3 $1
    TAG_NAME=$RET_VAL
    GPL_TMP_FILE="fcivl_${TAG_NAME}.gpl"
    fwrite_gpl_head $GPL_TMP_FILE
    fprint $GPL_TMP_FILE "set title \"${TAG_NAME}\" \n"
    get_player_name_by_num $3 "0"
    PLAYER_NAME=$RET_VAL
    fprint $GPL_TMP_FILE "plot \"fcivl_${TAG_NAME}_0.txt\" using 2:5  title \"$PLAYER_NAME\" "
    for i in `seq 1 $2`
    do
	get_player_name_by_num $3 "$i"
	PLAYER_NAME=$RET_VAL
	fprint $GPL_TMP_FILE ", \"fcivl_${TAG_NAME}_$i.txt\" using 2:5  title \"$PLAYER_NAME\""
    done
    fprint $GPL_TMP_FILE "\n"
    fprint $GPL_TMP_FILE "pause -1 \"press ENTER\" \n"
}

# Args: (in_file, tag_num, max_player)
create_tag_graph(){
    get_tag_by_num $1 $2
    TAG_NAME=$RET_VAL
    create_txt_files $1 $2 $TAG_NAME
    create_gpl_file $2 $3 $1
}

# Args: (in_file, max_player)
create_all(){
    get_max_tag_num $1
    MAX_TAG=$?
    for i in `seq 0 $MAX_TAG`
    do
	create_tag_graph $1 $i $2
	echo -e -n "."
    done
    echo -e -n "\n"
}

# ----------------- MAIN:

IN_FILE="civscore.log"
OUT_FILE="fcivl.gpl"

if [ $# -eq 0 ]
then
    echo ""
    echo "Scorelog converter for freeciv (ver >=1.14.1) !"
    echo "Written by Dymitr Pszenicyn"
    echo "e-mail: d.pszenicyn@students.mimuw.edu.pl"
    echo ""
    echo "This program converts freeciv score log to gnuplot readable format."
    echo "You can view your scores by writing: gnuplot fcivl.gpl"
    echo ""
    echo "Usage: $0 <input_civscore.log_file> [<output_fcivl.gpl_file>]"
    echo "Default out_file is fcivl.gpl"
    exit 1
fi

IN_FILE=$1

if [ $# -ge 2 ]
then
    OUT_FILE=$2
fi

if [ ! -r $IN_FILE ]
then
    echo "Can't read from file $IN_FILE !!!!"
    exit 1
fi

if [ -a $OUT_FILE -a ! -w $OUT_FILE ]
then
    echo "Can't write to file $OUT_FILE !!!!"
    exit 1
fi

rm -f $OUT_FILE
rm -f fcivl_*.txt
rm -f fcivl_*.gpl

echo "Please wait, creating file $OUT_FILE ..."

get_max_player_num $IN_FILE
MAX_PLAYER=$?

create_all $IN_FILE $MAX_PLAYER

cat fcivl_*.gpl >$OUT_FILE

echo "Done."
echo "Now you can view scores by running gnuplot:  gnuplot $OUT_FILE"

# ----------------- END MAIN
